import * as wpj from '../support/wpj';

describe('Konfigurace multisetu a přidání do košíku', () => {
   it('Vejdi na detail multiset produktu', () => {
      cy.visit('multiset-konfigurator_z6163/');
   });

   it('Otevři výběr první části multisetu', () => {
      wpj.clickMultisetSelectionButton(1);
   });

   it('Vyber druhý produkt a variantu M', () => {
      wpj.selectMultisetItem(2);
      wpj.selectMultisetVariation('M');
   });

   it('Otevři popisek a galerii', () => {
      cy.get('.jsmultisets-opener-title-wrapper').click({ multiple: true });
      cy.wait(1000);
      // TODO(havlicek): Vymyslet, jak udělat dobrý screenshot bez duplikace multisetu nebo použití viewportu, který je hodně malý
      cy.matchImageSnapshot('Multiset s otevřeným popiskem a galerií', { capture: 'viewport' });
   });

   it('Vlož výběr do setu', () => {
      wpj.clickAddToSetButton();
   });

   it('Otevři výběr druhé části multisetu, vyber variantu XL a vlož do setu', () => {
      wpj.clickMultisetSelectionButton(2);
      wpj.selectMultisetVariation('M');
      wpj.clickAddToSetButton();
   });

   it('Otevři výběr třetí části multisetu a vlož do setu', () => {
      wpj.clickMultisetSelectionButton(3);
      wpj.clickAddToSetButton();
   });

   it('Přidej multiset do košíku', () => {
      cy.get('.product-right-side .c-product-addtocartbutton').click();
      cy.wait(1000);
      cy.get('.cartbox-product').contains('Multiset');
   });
});
